@@ -14,9 +14,6 @@
 
 #if INCLUDE_GNOME
  #include "gnome.h"
- #define POLYGON_RENDER_CALLBACK model_render_polygons_gc
-#else
- #define POLYGON_RENDER_CALLBACK model_render_polygons
 #endif
 
   // basic model
@@ -47,7 +44,7 @@
 
 extern stg_type_record_t typetable[];
 
-//extern int _stg_disable_gui;
+extern int _stg_disable_gui;
 
 /** @ingroup stage 
     @{ 
@@ -84,6 +81,7 @@ model
   blobfinder_return 1
   fiducial_return 1
   gripper_return 0
+  audio_return 0
 
   fiducial_key 0
 
@@ -116,11 +114,13 @@ model
   - specify the initial velocity of the model. Not that if the model hits an obstacle, its velocity will be set to zero.
 - color [colorname:string]
   - specify the color of the object using a color name from the X11 database (rgb.txt)
-- line_count [int]
-  - specify the number of lines that make up the model's body
-- line[index] [x1:float y1:float x2:float y2:float]
-  - creates a line from (x1,y1) to (x2,y2). A set of line_count lines defines the robot's body for the purposes of collision detection and rendering in the GUI window.
-- bitmap [filename:string}
+- polygons [int]
+  - specify the number of polygons that make up the model's body
+- polygon[index].points [int]
+  - specify the number of points in a polygon
+- polygon[index].point[index] [x:float y:float]
+  - creates a point on the polgon.
+- bitmap [filename:string]
   - alternative way to set the model's line_count and lines. The file must be a bitmap recognized by libgtkpixbuf (most popular formats are supported). The file is opened and parsed into a set of lines. Unless the bitmap_resolution option is used, the lines are scaled to fit inside the rectangle defined by the model's current size.
 - gui_nose [bool]
   - if 1, draw a nose on the model showing its heading (positive X axis)
@@ -146,6 +146,9 @@ model
    - iff 1, this model can be detected by a ranger.
 - gripper_return [bool]
    - iff 1, this model can be gripped by a gripper and can be pushed around by collisions with anything that has a non-zero obstacle_return.
+- audio_return [bool]
+   - if 1, this model will be an obstacle to audio and will be used to precalculate the audio paths.
+     <b>warning: don't use this for moving objects</b>
 
 */
 
@@ -207,6 +210,7 @@ void stg_model_add_callback( stg_model_t* mod,
 			     stg_model_callback_t cb, 
 			     void* user )
 {
+  // TODO: free it somewhere or don't allocate it (pooya)
   int* key = malloc(sizeof(int));
   *key = address - (void*)mod;
 
@@ -396,6 +400,7 @@ stg_model_t* stg_model_create( stg_world_t* world,
   mod->blob_return = 1;
   mod->laser_return = LaserVisible;
   mod->gripper_return = 0;
+  mod->audio_return = 0;
   mod->boundary = 0;
   mod->color = 0xFF0000; // red;  
   mod->map_resolution = 0.1; // meters
@@ -420,12 +425,23 @@ stg_model_t* stg_model_create( stg_world_t* world,
 /*   mod->lines = pline; */
 /*   mod->lines_count = 1; */
 
+  if (!_stg_disable_gui)
+  {
+
   // now it's safe to create the GUI components
   if( mod->world->win )
     gui_model_create( mod );
 
   // GUI callbacks - draw changes
 
+
+#if INCLUDE_GNOME
+  gc_model_init( mod );
+
+
+#else
+  stg_model_add_callback( mod, &mod->pose, gui_model_move, NULL );
+
   // changes in any of these properties require a redraw of the model
   stg_model_add_callback( mod, &mod->polygons, gui_model_polygons, NULL );
   stg_model_add_callback( mod, &mod->color, gui_model_polygons, NULL );
@@ -437,30 +453,35 @@ stg_model_t* stg_model_create( stg_world_t* world,
 
   // these changes can be handled without a complete redraw
   stg_model_add_callback( mod, &mod->gui_grid, gui_model_grid, NULL );
-  stg_model_add_callback( mod, &mod->pose, gui_model_move, NULL );
   stg_model_add_callback( mod, &mod->gui_mask, gui_model_mask, NULL );
   stg_model_add_callback( mod, &mod->parent, gui_model_mask, NULL );
+#endif
 
   // now we can add the basic square shape
   stg_polygon_t* square = stg_unit_polygon_create();
+  
+  mod->polygons = NULL; /* initialize to NULL to prevent incidental delete by stg_model_set_polygons() below. */
+  mod->polygons_count = 0;
+
   stg_model_set_polygons( mod, square, 1 );
 
-#if INCLUDE_GNOME
-  GnomeCanvasGroup* parent_grp =
-    mod->parent ? mod->parent->grp : gnome_canvas_root( mod->world->win->gcanvas );
+/* #if INCLUDE_GNOME */
+/*   GnomeCanvasGroup* parent_grp = */
+/*     mod->parent ? mod->parent->grp : gnome_canvas_root( mod->world->win->gcanvas ); */
   
-  mod->grp = GNOME_CANVAS_GROUP(
-    gnome_canvas_item_new( parent_grp,
-			   gnome_canvas_group_get_type(),
-			   "x", pose.x,
-			   "y", pose.y,
-			   NULL ));
+/*   mod->grp = GNOME_CANVAS_GROUP( */
+/*     gnome_canvas_item_new( parent_grp, */
+/* 			   gnome_canvas_group_get_type(), */
+/* 			   "x", mod->pose.x, */
+/* 			   "y", mod->pose.y, */
+/* 			   NULL )); */
 
-  gnome_canvas_item_raise_to_top( GNOME_CANVAS_ITEM(mod->grp) );
-#endif
+/*   gnome_canvas_item_raise_to_top( GNOME_CANVAS_ITEM(mod->grp) ); */
+/* #endif */
 
   // exterimental: creates a menu of models
-  gui_add_tree_item( mod );
+  // gui_add_tree_item( mod );
+  }
   
   PRINT_DEBUG3( "finished model %d.%d(%s)", 
 		mod->world->id, 
@@ -471,6 +492,7 @@ stg_model_t* stg_model_create( stg_world_t* world,
   if( mod->typerec->initializer )
     mod->typerec->initializer(mod);
   
+#if ! INCLUDE_GNOME
   stg_model_add_property_toggles( mod, 
 				  &mod->velocity,
  				  model_render_velocity, // called when toggled on
@@ -480,7 +502,7 @@ stg_model_t* stg_model_create( stg_world_t* world,
 				  "velocityvector",
  				  "velocity vector",
 				  FALSE );
-
+#endif
 
 
   return mod;
@@ -489,7 +511,7 @@ stg_model_t* stg_model_create( stg_world_t* world,
 // free the memory allocated for a model
 void stg_model_destroy( stg_model_t* mod )
 {
-  assert( mod );
+  assert( mod != NULL );
   
   // remove from parent, if there is one
   if( mod->parent && mod->parent->children ) g_ptr_array_remove( mod->parent->children, mod );
@@ -953,6 +975,13 @@ void stg_model_set_gripper_return( stg_model_t* mod, int val )
   model_change( mod, &mod->gripper_return );
 }
 
+void stg_model_set_audio_return( stg_model_t* mod, int val )
+{
+  assert(mod);
+  mod->audio_return = val;
+  model_change( mod, &mod->audio_return );
+}
+
 void stg_model_set_fiducial_return( stg_model_t* mod, int val )
 {
   assert(mod);
@@ -1068,14 +1097,18 @@ void stg_model_set_polygons( stg_model_t* mod,
   if( poly_count == 0 )
     {
       stg_model_map( mod, 0 ); // unmap the model from the matrix
-      
-      free( mod->polygons );
-      mod->polygons_count = 0;      
+
+      if ( mod->polygons != NULL )
+      {      
+	stg_polygons_destroy( mod->polygons, mod->polygons_count );
+	mod->polygons = NULL;
+	mod->polygons_count = 0;
+      }
     }
   else
     {
-      assert(polys);
-      
+      assert(polys != NULL);
+
       stg_model_map( mod, 0 ); // unmap the model from the matrix
       
       // normalize the polygons to fit exactly in the model's body
@@ -1087,18 +1120,18 @@ void stg_model_set_polygons( stg_model_t* mod,
       //else
       //PRINT_WARN3( "setting polygons for model %s which has non-positive area (%.2f,%.2f)",
       //	     prop->mod->token, geom.size.x, geom.size.y );
-      
+
       // zap our old polygons
-      stg_polygons_destroy( mod->polygons, mod->polygons_count );
-      
+      if ( mod->polygons != NULL )
+	stg_polygons_destroy( mod->polygons, mod->polygons_count );
+
       mod->polygons = polys;
       mod->polygons_count = poly_count;
-      
+     
       // if the model has some non-zero
       stg_model_map( mod, 1 ); // map the model into the matrix with the new polys
     }
 
-  
   model_change( mod, &mod->polygons );
 }
 
@@ -1396,7 +1429,7 @@ int stg_model_update_pose( stg_model_t* mod )
 	  interval = 0.2 * interval; // slow down time
 	  
 	  // inch forward until we collide
-	  do
+	  for (int i=0; i<5; i++)
 	    {
 	      memcpy( &old_pose, &mod->pose, sizeof(old_pose));
 	      
@@ -1408,7 +1441,9 @@ int stg_model_update_pose( stg_model_t* mod )
 	      
 	      hitthing = stg_model_test_collision( mod, &hitx, &hity );
 	      
-	    } while( hitthing == NULL );
+	      if (hitthing != NULL)
+	        break;
+	    }
 	  
 	  //PRINT_WARN( "HIT something immovable!" );
 	  
@@ -1425,8 +1460,10 @@ int stg_model_update_pose( stg_model_t* mod )
 	}
     }
 
-  
-  stg_model_set_pose( mod, &mod->pose );
+  // if the pose changed, we need to set it.
+  if( memcmp( &old_pose, &mod->pose, sizeof(old_pose) ))
+      stg_model_set_pose( mod, &mod->pose );
+
   stg_model_set_stall( mod, stall );
   
   /* trails */
@@ -1567,81 +1604,90 @@ int model_render_velocity( stg_model_t* mod, void* enabled )
 }
 
 
-int stg_model_set_named_property_int( stg_model_t* mod, 
-				      char* name, 
-				      size_t len, 
-				      int value )
+//#define ISPROP(A,B) (strncmp( A, B, strlen(B) == 0 ))
+
+int ISPROP( char* name, char* match )
 {
-  int ack = FALSE;
+  return( strcmp( name, match ) == 0 );
+}
+
+int stg_model_set_property_named( stg_model_t* mod, 
+				  char* name, 
+				  void* value )
+{
+  PRINT_MSG1( "Looking up named property \"%s\"\n", name );
   
-  if( strncmp( name, "fiducial_return", len) == 0 )
+  if( ISPROP(name,STG_MP_FIDUCIAL_RETURN) )
     {
-      stg_model_set_fiducial_return( mod, value );
-      ack = TRUE;
+      stg_model_set_fiducial_return( mod, *(int*)value );
+      return 0;
     }
-  else if( strncmp( name, "laser_return", len) == 0 )
+  if( ISPROP( name, STG_MP_LASER_RETURN ) )
     {
-      stg_model_set_laser_return( mod, value );
-      ack = TRUE;
+      stg_model_set_laser_return( mod, *(int*)value );
+      return 0;
     }
-  else if( strncmp( name, "obstacle_return", len) == 0 )
+  if( ISPROP( name, STG_MP_OBSTACLE_RETURN ) )
     {
-      stg_model_set_obstacle_return( mod, value );
-	      ack = TRUE;
+      stg_model_set_obstacle_return( mod, *(int*)value );
+      return 0;
     }
-  else if( strncmp( name, "ranger_return", len) == 0 )
+  if( ISPROP( name, STG_MP_RANGER_RETURN ) )
     {
-      stg_model_set_ranger_return( mod, value );
-      ack = TRUE;
+      stg_model_set_ranger_return( mod, *(int*)value );
+      return 0;
     }
-  else if( strncmp( name, "gripper_return", len) == 0 )
+  if( ISPROP( name, STG_MP_GRIPPER_RETURN ) )
     {
-      stg_model_set_gripper_return( mod, value );
-      ack = TRUE;
+      stg_model_set_gripper_return( mod, *(int*)value );
+      return 0;
     }
-  else if( strncmp( name, "color", len) == 0 )
+  if( ISPROP( name, STG_MP_AUDIO_RETURN ) )
     {
-      stg_model_set_color( mod, value );
-      ack = TRUE;
+      stg_model_set_audio_return( mod, *(int*)value );
+      return 0;
+    }
+  if( ISPROP( name, STG_MP_COLOR ) )
+    {
+      stg_model_set_color( mod, *(int*)value );
+      return 0;
+    }
+  if( ISPROP( name, STG_MP_MASS ) )
+    {
+      stg_model_set_mass( mod, *(double*)value );
+      return 0;
+    }
+  if( ISPROP( name, STG_MP_WATTS ) )
+    {
+      stg_model_set_watts( mod, *(double*)value );
+      return 0;
     }
-  else
-    printf( "Stage: failed to set unknown integer property \"%s\"\n",
-	    name );
   
-  return ack;
+  PRINT_ERR1( "Unknown property name with system prefix \"%s\"\n", name );
+  return 1; // error code
 }
 
 
-int stg_model_set_named_property_double( stg_model_t* mod, 
-					 char* name, 
-					 size_t len, 
-					 double value )
-{
-  int ack = FALSE;
-  
-  if( strncmp( name, "mass", len) == 0 )
-    {
-      stg_model_set_mass( mod, value );
-      ack = TRUE;
-    }
-  else if( strncmp( name, "watts", len) == 0 )
-    {
-      stg_model_set_watts( mod, value );
-      ack = TRUE;
-    }
-  else
-    printf( "Stage: failed to set unknown floating-point property \"%s\"\n",
-	    name );
-  
-  return ack;
-}
-
-
-void stg_model_set_property( stg_model_t* mod,
+int stg_model_set_property( stg_model_t* mod,
 			     char* key,
-			     void* data,
-			     size_t len )
+			    void* data )
 {
+  // see if the key has the predefined-property prefix
+  if( strncmp( key, STG_MP_PREFIX, strlen(STG_MP_PREFIX)) == 0 )
+    return stg_model_set_property_named( mod, key, data );
+  else
+    g_datalist_set_data( &mod->props, key, data );
+  
+  return 0; // ok
 }
 
 
+void stg_model_unset_property( stg_model_t* mod, char* key )
+{
+  g_datalist_remove_data( &mod->props, key );
+}
+
+void* stg_model_get_property( stg_model_t* mod, char* key )
+{
+  return g_datalist_get_data( &mod->props, key );
+}
