@@ -5,9 +5,9 @@
 // Date: 10 June 2004
 //
 // CVS info:
-//  $Source: /cvsroot/playerstage/code/stage/src/model_position.c,v $
+//  $Source: /home/tcollett/stagecvs/playerstage-cvs/code/stage/src/model_position.c,v $
 //  $Author: gerkey $
-//  $Revision: 1.60 $
+//  $Revision: 1.62 $
 //
 ///////////////////////////////////////////////////////////////////////////
 
@@ -54,10 +54,12 @@ position
   drive "diff"
   localization "gps"
   localization_origin [ <defaults to model's start pose> ]
+  watchdog_timeout -1.0
  
   # odometry error model parameters, 
   # only used if localization is set to "odom"
-  odom_error [0.03 0.03 0.05]
+  # see description below for what the parameters mean
+  odom_error [0.01 0.05 0.01 0.02 0.01 0.02]
 
   # model properties
 )
@@ -73,8 +75,10 @@ Since Stage-1.6.5 the odom property has been removed. Stage will generate a warn
   - if "gps" the position model reports its position with perfect accuracy. If "odom", a simple odometry model is used and position data drifts from the ground truth over time. The odometry model is parameterized by the odom_error property.
 - localization_origin [x y theta]
   - set the origin of the localization coordinate system. By default, this is copied from the model's initial pose, so the robot reports its position relative to the place it started out. Tip: If localization_origin is set to [0 0 0] and localization is "gps", the model will return its true global position. This is unrealistic, but useful if you want to abstract away the details of localization. Be prepared to justify the use of this mode in your research! 
-- odom_error [x y theta]
-  - parameters for the odometry error model used when specifying localization "odom". Each value is the maximum proportion of error in intergrating x, y, and theta velocities to compute odometric position estimate. For each axis, if the the value specified here is E, the actual proportion is chosen at startup at random in the range -E/2 to +E/2. Note that due to rounding errors, setting these values to zero does NOT give you perfect localization - for that you need to choose localization "gps".
+- odom_error [x xstd  y ystd  theta thetastd]
+  - parameters for the odometry error model used when specifying localization "odom". Defines a gaussian error model with a mean and standard deviation.  A nonzero mean introduces a bias into the odometry, which is realistic for many robots.  The standard deviations are with respect to a standard length (1m) for x and y, and with respect to a full revolution for theta.  The exact interpretation of the parameters depends on the drive model.  For differential drive, theta and y contribute to angle errors, based on the amount of turn (theta) and amount of distance traveled (y).  Translational distance error is given by x.  For other drive models, only the bias term is used.  Note that due to rounding errors, setting these values to zero does NOT give you perfect localization - for that you need to choose localization "gps".
+- watchdog_timeout [float]
+  - time, in seconds, since the last command was received after which the robot will be stopped.  Set to -1.0 for no timeout (this is the default).
 */
 
 
@@ -110,11 +114,33 @@ void stg_model_position_set_odom( stg_model_t* mod, stg_pose_t* odom )
 const double STG_POSITION_WATTS_KGMS = 5.0; // cost per kg per meter per second
 const double STG_POSITION_WATTS = 2.0; // base cost of position device
 
-// simple odometry error model parameters. the error is selected at
+// simple odometry error model parameters. The bias is selected at
 // random in the interval -MAX/2 to +MAX/2 at startup
-const double STG_POSITION_INTEGRATION_ERROR_MAX_X = 0.03;
-const double STG_POSITION_INTEGRATION_ERROR_MAX_Y = 0.03;
-const double STG_POSITION_INTEGRATION_ERROR_MAX_A = 0.05;
+// std is set to a fixed value
+const double STG_POSITION_INTEGRATION_BIAS_MAX_X = 0.03;
+const double STG_POSITION_INTEGRATION_BIAS_MAX_Y = 0.03;
+const double STG_POSITION_INTEGRATION_BIAS_MAX_A = 0.05;
+const double STG_POSITION_INTEGRATION_ERROR_X = 0.03;
+const double STG_POSITION_INTEGRATION_ERROR_Y = 0.03;
+const double STG_POSITION_INTEGRATION_ERROR_A = 0.05;
+
+// return two random number from a zero-mean normalized gaussian
+// distribution.
+// to get mean A and std B, use A*y + B
+double drand48();
+void gauss_rand(double *y1, double *y2)
+{
+  double x1, x2, w;
+  do {
+    x1 = 2.0 * drand48() - 1.0;
+    x2 = 2.0 * drand48() - 1.0;
+    w = x1 * x1 + x2 * x2;
+  } while ( w >= 1.0 );
+
+  w = sqrt( (-2.0 * log( w ) ) / w );
+  *y1 = x1 * w;
+  *y2 = x2 * w;
+}
 
 int position_startup( stg_model_t* mod );
 int position_shutdown( stg_model_t* mod );
@@ -168,22 +194,34 @@ void position_init( stg_model_t* mod )
   stg_position_data_t data;
   memset( &data, 0, sizeof(data));
   
-  data.integration_error.x =  
-    drand48() * STG_POSITION_INTEGRATION_ERROR_MAX_X - 
-    STG_POSITION_INTEGRATION_ERROR_MAX_X/2.0;
+  data.integration_bias.x =  
+    drand48() * STG_POSITION_INTEGRATION_BIAS_MAX_X - 
+    STG_POSITION_INTEGRATION_BIAS_MAX_X/2.0;
   
-  data.integration_error.y =  
-    drand48() * STG_POSITION_INTEGRATION_ERROR_MAX_Y - 
-    STG_POSITION_INTEGRATION_ERROR_MAX_Y/2.0;
+  data.integration_error.x = STG_POSITION_INTEGRATION_ERROR_X;
+
+  data.integration_bias.y =  
+    drand48() * STG_POSITION_INTEGRATION_BIAS_MAX_Y - 
+    STG_POSITION_INTEGRATION_BIAS_MAX_Y/2.0;
   
-  data.integration_error.a =  
-    drand48() * STG_POSITION_INTEGRATION_ERROR_MAX_A - 
-    STG_POSITION_INTEGRATION_ERROR_MAX_A/2.0;
+  data.integration_error.y = STG_POSITION_INTEGRATION_ERROR_Y;
+
+  data.integration_bias.a =  
+    drand48() * STG_POSITION_INTEGRATION_BIAS_MAX_A - 
+    STG_POSITION_INTEGRATION_BIAS_MAX_A/2.0;
   
+  data.integration_error.a = STG_POSITION_INTEGRATION_ERROR_A;
+
+
   data.localization = STG_POSITION_LOCALIZATION_DEFAULT;
+
+  data.watchdog_timeout = -1;
   
+  data.wheelbase = 1.0;
+
   stg_model_set_data( mod, &data, sizeof(data));
   
+#if ! INCLUDE_GNOME
   stg_model_add_property_toggles( mod, 
 				  &mod->data,
  				  position_render_data, // called when toggled on
@@ -203,6 +241,7 @@ void position_init( stg_model_t* mod )
 				  "positiontext",
  				  "position text",
 				  FALSE );
+#endif
 }
 
 
@@ -280,15 +319,54 @@ void position_load( stg_model_t* mod )
       memset( &data->pose_error, 0, sizeof(data->pose_error));      
     }
 
+  if( wf_property_exists( mod->id, "max_speed" ) )
+    {
+      data->max_speed.x = wf_read_tuple_length(mod->id, "max_speed", 0, data->max_speed.x );
+      data->max_speed.y = wf_read_tuple_length(mod->id, "max_speed", 1, data->max_speed.y );
+      data->max_speed.a = wf_read_tuple_angle(mod->id, "max_speed", 2, data->max_speed.a );
+    }
+
+  // did the user specify a wheelbase
+  if( wf_property_exists( mod->id, "wheelbase" ) )
+    {
+      data->wheelbase = 
+	wf_read_float( mod->id, "wheelbase", 1.0 );
+    }
+
   // odometry model parameters
   if( wf_property_exists( mod->id, "odom_error" ) )
     {
-      data->integration_error.x = 
-	wf_read_tuple_length(mod->id, "odom_error", 0, data->integration_error.x );
-      data->integration_error.y = 
-	wf_read_tuple_length(mod->id, "odom_error", 1, data->integration_error.y );
-      data->integration_error.a 
-	= wf_read_tuple_angle(mod->id, "odom_error", 2, data->integration_error.a );
+      if (wf_tuple_count( mod->id, "odom_error" ) == 3)
+      // old model, just use bias
+      {
+        data->integration_bias.x = 
+          wf_read_tuple_length(mod->id, "odom_error", 0, data->integration_bias.x );
+        data->integration_error.x = 0;
+        data->integration_bias.y = 
+          wf_read_tuple_length(mod->id, "odom_error", 1, data->integration_bias.y );
+        data->integration_error.y = 0;
+        data->integration_bias.a = 
+          wf_read_tuple_angle(mod->id, "odom_error", 2, data->integration_bias.a );
+        data->integration_error.a = 0;
+      }
+      else if (wf_tuple_count( mod->id, "odom_error" ) == 6)
+      {
+        data->integration_bias.x = 
+          wf_read_tuple_length(mod->id, "odom_error", 0, data->integration_bias.x );
+        data->integration_error.x = 
+          wf_read_tuple_length(mod->id, "odom_error", 1, data->integration_error.x );
+        data->integration_bias.y = 
+          wf_read_tuple_length(mod->id, "odom_error", 2, data->integration_bias.y );
+        data->integration_error.y = 
+          wf_read_tuple_length(mod->id, "odom_error", 3, data->integration_error.y );
+        data->integration_bias.a = 
+          wf_read_tuple_angle(mod->id, "odom_error", 4, data->integration_bias.a );
+        data->integration_error.a = 
+          wf_read_tuple_angle(mod->id, "odom_error", 5, data->integration_error.a );
+      }
+      else
+      PRINT_ERR1( "Odometry error tuple must be 3 (bias) or 6 (bias+std) long in model \"%s\".",
+                  mod->token );
     }
 
   // choose a localization model
@@ -313,6 +391,13 @@ void position_load( stg_model_t* mod )
 		    mod->token );
     }
   
+  // did the user specify a watchdog timeout?
+  if( wf_property_exists( mod->id, "watchdog_timeout" ) )
+    {
+      data->watchdog_timeout = 
+              (int)rint(1e3 * wf_read_float( mod->id, "watchdog_timeout", -1.0 ));
+    }
+  
   // we've probably poked the localization data, so we must let people know
   model_change( mod, &mod->data );
 }
@@ -338,6 +423,52 @@ int position_update( stg_model_t* mod )
     {            
       switch( cmd->mode )
 	{
+	case STG_POSITION_CONTROL_VELOCITY_HEADING:
+	  // mixed-mode control: translational velocity, rotational heading
+
+	  PRINT_DEBUG( "velocity-heading control mode" );
+	  PRINT_DEBUG4( "model %s command(%.2f %.2f %.2f)",
+			mod->token, 
+			mod->cmd.x, 
+			mod->cmd.y, 
+			mod->cmd.a );
+
+	  double a_error = NORMALIZE( cmd->a - data->pose.a );
+	    
+	  PRINT_DEBUG3( "errors: %.2f %.2f %.2f\n", x_error, y_error, a_error );
+	    
+	  // speed limits for controllers
+	  // TODO - have these configurable
+	  double max_speed_a = 1.0; // rads/sec
+
+	  switch( cfg->drive_mode )
+	    {
+	    case STG_POSITION_DRIVE_DIFFERENTIAL:
+	      // differential-steering model, like a Pioneer
+		vel->x = cmd->x;
+		vel->y = 0;
+		vel->a = MIN( a_error, max_speed_a );
+		break;
+		
+	      case STG_POSITION_DRIVE_OMNI:
+		// direct steering model, like an omnidirectional robot
+		vel->x = cmd->x;
+		vel->y = cmd->y;
+		vel->a = MIN( a_error, max_speed_a );
+		break;
+
+	      case STG_POSITION_DRIVE_CAR:
+		// car like steering model, same as differential here
+		vel->x = cmd->x * cos(cmd->a);
+		vel->y = 0;
+		vel->a = MIN( a_error, max_speed_a );
+		break;
+
+	      default:
+		PRINT_ERR1( "unknown steering mode %d", cfg->drive_mode );
+	    }
+	  break;
+
 	case STG_POSITION_CONTROL_VELOCITY :
 	  {
 	    PRINT_DEBUG( "velocity control mode" );
@@ -368,7 +499,7 @@ int position_update( stg_model_t* mod )
 		// car like steering model based on speed and turning angle
 		vel->x = cmd->x * cos(cmd->a);
 		vel->y = 0;
-		vel->a = cmd->x * sin(cmd->a)/1.0; // here 1.0 is the wheel base, this should be a config option
+		vel->a = cmd->x * sin(cmd->a)/data->wheelbase; 
 		break;
 		
 	      default:
@@ -386,12 +517,6 @@ int position_update( stg_model_t* mod )
 	    
 	    PRINT_DEBUG3( "errors: %.2f %.2f %.2f\n", x_error, y_error, a_error );
 	    
-	    // speed limits for controllers
-	    // TODO - have these configurable
-	    double max_speed_x = 0.4;
-	    double max_speed_y = 0.4;
-	    double max_speed_a = 1.0;	      
-	    
 	    switch( cfg->drive_mode )
 	      {
 	      case STG_POSITION_DRIVE_OMNI:
@@ -399,9 +524,9 @@ int position_update( stg_model_t* mod )
 		  // this is easy - we just reduce the errors in each axis
 		  // independently with a proportional controller, speed
 		  // limited
-		  vel->x = MIN( x_error, max_speed_x );
-		  vel->y = MIN( y_error, max_speed_y );
-		  vel->a = MIN( a_error, max_speed_a );
+		  vel->x = MIN( x_error, data->max_speed.x );
+		  vel->y = MIN( y_error, data->max_speed.y );
+		  vel->a = MIN( a_error, data->max_speed.a );
 		}
 		break;
 		
@@ -425,8 +550,8 @@ int position_update( stg_model_t* mod )
 		    {
 		      PRINT_DEBUG( "TURNING ON THE SPOT" );
 		      // turn on the spot to minimize the error
-		      calc.a = MIN( a_error, max_speed_a );
-		      calc.a = MAX( a_error, -max_speed_a );
+		      calc.a = MIN( a_error, data->max_speed.a );
+		      calc.a = MAX( a_error, -data->max_speed.a );
 		    }
 		  else
 		    {
@@ -436,8 +561,8 @@ int position_update( stg_model_t* mod )
 		      double goal_distance = hypot( y_error, x_error );
 		      
 		      a_error = NORMALIZE( goal_angle - data->pose.a );
-		      calc.a = MIN( a_error, max_speed_a );
-		      calc.a = MAX( a_error, -max_speed_a );
+		      calc.a = MIN( a_error, data->max_speed.a );
+		      calc.a = MAX( a_error, -data->max_speed.a );
 		      
 		      PRINT_DEBUG2( "steer errors: %.2f %.2f \n", a_error, goal_distance );
 		      
@@ -446,7 +571,7 @@ int position_update( stg_model_t* mod )
 		      if( fabs(a_error) < M_PI/16 )
 			{
 			  PRINT_DEBUG( "DRIVING TOWARDS THE GOAL" );
-			  calc.x = MIN( goal_distance, max_speed_x );
+			  calc.x = MIN( goal_distance, data->max_speed.x );
 			}
 		    }
 		  
@@ -512,14 +637,43 @@ int position_update( stg_model_t* mod )
       {
 	// integrate our velocities to get an 'odometry' position estimate.
 	double dt = mod->world->sim_interval/1e3;
-	
-	data->pose.a = NORMALIZE( data->pose.a + (vel->a * dt) * (1.0 +data->integration_error.a) );
+      double oda = vel->a*dt;
+      double odx = vel->x*dt;
+      double ody = vel->y*dt;
+      double dx = 0;
+      double dy = 0;
+
+      // differential drive odometry error model
+      if (cmd->mode == STG_POSITION_CONTROL_VELOCITY &&
+          cfg->drive_mode == STG_POSITION_DRIVE_DIFFERENTIAL)
+
+        {
+          double e1, e2;
+          gauss_rand(&e1, &e2);
+          double ooda = fabs(oda);
+          double oodx = fabs(odx);
+          double avar = data->integration_error.a * data->integration_error.a; // variance @ 1 rev (in rads)
+          double xvar = data->integration_error.x * data->integration_error.x; // variance @ 1 m (in m)
+          double yvar = data->integration_error.y * data->integration_error.y; // variance @ 1 m (in m)
+          
+          data->pose.a = NORMALIZE( data->pose.a + oda + 
+              ooda * data->integration_bias.a + oodx * data->integration_bias.y +
+              sqrt((ooda/(2*M_PI))*avar + oodx*yvar)*e1);
+
+          dx = odx + oodx*data->integration_bias.x +
+                     sqrt(oodx*xvar)*e2;
+        }
+
+      else                    // not a diff drive error model
+        {
+          data->pose.a = NORMALIZE( data->pose.a + oda * (1.0 +data->integration_bias.a) );
+          dx = odx * (1.0 + data->integration_bias.x );
+          dy = ody * (1.0 + data->integration_bias.y );
+        }
 	
 	double cosa = cos(data->pose.a);
 	double sina = sin(data->pose.a);
-	double dx = (vel->x * dt) * (1.0 + data->integration_error.x );
-	double dy = (vel->y * dt) * (1.0 + data->integration_error.y );
-	
+		
 	data->pose.x += dx * cosa + dy * sina; 
 	data->pose.y -= dy * cosa - dx * sina;
 	model_change( mod, &mod->data );
