@@ -22,7 +22,7 @@
  * Desc: A plugin driver for Player that gives access to Stage devices.
  * Author: Richard Vaughan
  * Date: 10 December 2004
- * CVS: $Id: p_driver.cc,v 1.37 2006/03/29 05:11:00 rtv Exp $
+ * CVS: $Id: p_driver.cc,v 1.43 2007-12-11 23:23:48 gerkey Exp $
  */
 
 // DOCUMENTATION ------------------------------------------------------------
@@ -150,7 +150,6 @@ The stage plugin driver provides the following device interfaces:
 #include <math.h>
 
 #include "p_driver.h"
-#include "zoo_driver.h"
 
 #define STG_DEFAULT_WORLDFILE "default.world"
 #define DRIVER_ERROR(X) printf( "Stage driver error: %s\n", X )
@@ -205,7 +204,6 @@ int player_driver_init(DriverTable* table)
 {
   puts(" Stage driver plugin init");
   StgDriver_Register(table);
-  //ZooDriver_Register(table);
   return(0);
 }
 
@@ -297,12 +295,15 @@ InterfaceModel::InterfaceModel(  player_devaddr_t addr,
 }      
 
 
+// initialize a static member
+unsigned int StgDriver::instance_count = 0;
+
+extern int _stg_disable_gui;
 
 // Constructor.  Retrieve options from the configuration file and do any
 // pre-Setup() setup.
 
 // configure the underlying driver to queue incoming commands and use a very long queue.
-
 StgDriver::StgDriver(ConfigFile* cf, int section)
     : Driver(cf, section, false, 4096 )
 {  
@@ -382,8 +383,8 @@ StgDriver::StgDriver(ConfigFile* cf, int section)
 	  break;	  
 	  
 	case PLAYER_GRAPHICS2D_CODE:
-	  ifsrc = new InterfaceGraphics2d( player_addr,  this, cf, section );
-	  break;	  
+      ifsrc = new InterfaceGraphics2d( player_addr,  this, cf, section );
+      break;
 
 	case PLAYER_GRIPPER_CODE:
 	  ifsrc = new InterfaceGripper( player_addr,  this, cf, section );
@@ -397,9 +398,17 @@ StgDriver::StgDriver(ConfigFile* cf, int section)
 	  ifsrc = new InterfaceSpeech( player_addr,  this, cf, section );
 	  break;	  
 
+	case PLAYER_OPAQUE_CODE:
+	  ifsrc = new InterfaceAudio( player_addr,  this, cf, section );
+	  break;	  
+
 	case PLAYER_BUMPER_CODE:
 	  ifsrc = new InterfaceBumper( player_addr,  this, cf, section );
 	  break;	  
+	  
+	case PLAYER_DIO_CODE:
+	  ifsrc = new InterfaceDio( player_addr,  this, cf, section );
+	  break;
 
 	default:
 	  PRINT_ERR1( "error: stage driver doesn't support interface type %d\n",
@@ -468,6 +477,7 @@ stg_model_t*  StgDriver::LocateModel( char* basename,
 // Set up the device.  Return 0 if things go well, and -1 otherwise.
 int StgDriver::Setup()
 {   
+  StgDriver::instance_count++;
   //puts("stage driver setup");  
   return(0);
 }
@@ -492,39 +502,45 @@ Interface* StgDriver::LookupDevice( player_devaddr_t addr )
 
 
 // subscribe to a device
-int StgDriver::Subscribe(player_devaddr_t addr)
+int StgDriver::Subscribe(QueuePointer &queue, player_devaddr_t addr)
 {
   if( addr.interf == PLAYER_SIMULATION_CODE )
-    return 0; // ok
+    return Driver::Subscribe(addr);
 
   Interface* device = this->LookupDevice( addr );
   
   if( device )
     {      
       stg_model_subscribe( device->mod );  
-      return Driver::Subscribe(addr);
+      int result = Driver::Subscribe(addr);
+      if ( result != 0 )
+    	  return result;
+      return device->Subscribe(queue, addr);
     }
 
   puts( "failed to find a device" );
-  return 1; // error
+  return -1; // error
 }
 
 
 // unsubscribe to a device
-int StgDriver::Unsubscribe(player_devaddr_t addr)
+int StgDriver::Unsubscribe(QueuePointer &queue, player_devaddr_t addr)
 {
   if( addr.interf == PLAYER_SIMULATION_CODE )
-    return 0; // ok
+    return Driver::Unsubscribe(addr);
 
   Interface* device = this->LookupDevice( addr );
   
   if( device )
     {
       stg_model_unsubscribe( device->mod );  
-      return Driver::Unsubscribe(addr);
+      int result = Driver::Unsubscribe(addr);
+      if ( result != 0 )
+    	  return result;
+      return device->Unsubscribe(queue, addr);
     }
   else
-    return 1; // error
+    return -1; // error
 }
 
 StgDriver::~StgDriver()
@@ -541,7 +557,7 @@ StgDriver::~StgDriver()
 // Shutdown the device
 int StgDriver::Shutdown()
 {
-  puts("Shutting stage driver down");
+  //puts("Shutting stage driver down");
 
   // Stop and join the driver thread
   // this->StopThread(); // todo - the thread only runs in the sim instance
@@ -553,7 +569,8 @@ int StgDriver::Shutdown()
   //  stg_model_unsubscribe( device->mod );
   // }
 
-  puts("stage driver has been shutdown");
+  if( --StgDriver::instance_count == 0 )
+    puts("All Stage drivers have been shutdown.");
 
   return(0);
 }
@@ -564,7 +581,7 @@ int StgDriver::Shutdown()
 // Driver::ProcessMessages() is called by StgDriver::Update(), which is
 // called periodically by player.
 int 
-StgDriver::ProcessMessage(MessageQueue* resp_queue, 
+StgDriver::ProcessMessage(QueuePointer &resp_queue, 
 			  player_msghdr * hdr, 
 			  void * data)
 {
@@ -610,7 +627,7 @@ void StgDriver::Update(void)
       double currtime;
       GlobalTime->GetTimeDouble(&currtime);
       if((currtime - interface->last_publish_time) >= 
-         (interface->mod->world->sim_interval / 1e3))
+         (interface->mod->world->sim_interval / 1e3 - 1e-9))
       {
         interface->Publish();
         interface->last_publish_time = currtime;
