@@ -11,6 +11,7 @@
 #include <math.h>
 #include <glib.h>
 #include <locale.h>
+#include <string.h>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
@@ -18,6 +19,7 @@
 
 #include "replace.h"
 #include "stage_internal.h"
+#include "config.h"
 
 int _stg_quit = FALSE;
 int _stg_disable_gui = FALSE;
@@ -108,15 +110,15 @@ stg_msec_t stg_timenow( void )
 {
   struct timeval tv;
   static stg_msec_t starttime = 0;
-  
+
   gettimeofday( &tv, NULL );
-  
+
   stg_msec_t timenow = (stg_msec_t)( tv.tv_sec*1000 + tv.tv_usec/1000 );
-  
-  
+
+
   if( starttime == 0 )
     starttime = timenow;
-  
+
   return( timenow - starttime );
 }
 
@@ -142,59 +144,84 @@ void stg_quit_cancel( void )
 // red color will be returned instead.
 stg_color_t stg_lookup_color(const char *name)
 {
-  FILE *file;
-  const char *filename;
-  
   if( name == NULL ) // no string?
     return 0; // black
-  
+
   if( strcmp( name, "" ) == 0 ) // empty string?
     return 0; // black
 
-  filename = COLOR_DATABASE;
-  file = fopen(filename, "r");
-  if (!file)
-  {
-    PRINT_ERR2("unable to open color database %s : %s",
-               filename, strerror(errno));
-    fclose(file);
-    return 0xFFFFFF;
-  }
-  
-  while (TRUE)
-  {
-    char line[1024];
-    if (!fgets(line, sizeof(line), file))
-      break;
+  static FILE *file = NULL;
+  static GHashTable* table = NULL;
 
-    // it's a macro or comment line - ignore the line
-    if (line[0] == '!' || line[0] == '#' || line[0] == '%') 
-      continue;
+  if( table == NULL )
+    table = g_hash_table_new( g_str_hash, g_str_equal );
 
-    // Trim the trailing space
-    while (strchr(" \t\n", line[strlen(line)-1]))
-      line[strlen(line)-1] = 0;
-
-    // Read the color
-    int r, g, b;
-    int chars_matched = 0;
-    sscanf( line, "%d %d %d %n", &r, &g, &b, &chars_matched );
-      
-    // Read the name
-    char* nname = line + chars_matched;
-
-    // If the name matches
-    if (strcmp(nname, name) == 0)
+  if( file == NULL )
     {
-      fclose(file);
-      return ((r << 16) | (g << 8) | b);
-    }
-  }
-  PRINT_WARN1("unable to find color [%s]; using default (red)", name);
-  fclose(file);
-  return 0xFF0000;
-}
+      char* searchfiles[] = {
+	"./rgb.txt",
+#ifdef COLOR_DATABASE
+	COLOR_DATABASE,
+#endif
+	"../rgb.txt",
+	NULL };
 
+      for( int i=0;
+	   searchfiles[i];
+	   i++ )
+	{
+	  char* filename = searchfiles[i];
+	  PRINT_DEBUG1( "Attempting to open \"%s\"", filename );
+	  if( (file = fopen( filename, "r")) )
+	    break; // opened a file ok - jump out of for loop
+	}
+
+      if( file == NULL )
+	{
+
+	  PRINT_ERR1("unable to open color database: %s",
+		     strerror(errno));
+	  exit(0);
+	}
+
+      PRINT_DEBUG( "Success!" );
+
+      // load the file into the hash table
+      while (TRUE)
+	{
+	  char line[1024];
+	  if (!fgets(line, sizeof(line), file))
+	    break;
+
+	  // it's a macro or comment line - ignore the line
+	  if (line[0] == '!' || line[0] == '#' || line[0] == '%')
+	    continue;
+
+	  // Trim the trailing space
+	  while (strchr(" \t\n", line[strlen(line)-1]))
+	    line[strlen(line)-1] = 0;
+
+	  // Read the color
+	  int r, g, b;
+	  int chars_matched = 0;
+	  sscanf( line, "%d %d %d %n", &r, &g, &b, &chars_matched );
+
+	  stg_color_t col = ( 0xFF000000 | (r << 16) | (g << 8) | b);
+
+	  // Read the name
+	  char* colorname = strdup( line + chars_matched );
+
+	  // map the name to the color in the table
+	  g_hash_table_insert( table, (gpointer)colorname, (gpointer)col );
+
+	}
+
+      fclose(file);
+    }
+
+  // look up the colorname in the database
+  return (stg_color_t)g_hash_table_lookup( table, name );
+}
 
 
 //////////////////////////////////////////////////////////////////////////
@@ -205,28 +232,28 @@ void stg_lines_normalize( stg_line_t* lines, int num )
   double minx, miny, maxx, maxy;
   minx = miny = BILLION;
   maxx = maxy = -BILLION;
-  
+
   int l;
   for( l=0; l<num; l++ )
     {
       // find the bounding rectangle
       if( lines[l].x1 < minx ) minx = lines[l].x1;
-      if( lines[l].y1 < miny ) miny = lines[l].y1;      
-      if( lines[l].x1 > maxx ) maxx = lines[l].x1;      
+      if( lines[l].y1 < miny ) miny = lines[l].y1;
+      if( lines[l].x1 > maxx ) maxx = lines[l].x1;
       if( lines[l].y1 > maxy ) maxy = lines[l].y1;
       if( lines[l].x2 < minx ) minx = lines[l].x2;
-      if( lines[l].y2 < miny ) miny = lines[l].y2;      
-      if( lines[l].x2 > maxx ) maxx = lines[l].x2;      
+      if( lines[l].y2 < miny ) miny = lines[l].y2;
+      if( lines[l].x2 > maxx ) maxx = lines[l].x2;
       if( lines[l].y2 > maxy ) maxy = lines[l].y2;
     }
-  
+
   // now normalize all lengths so that the lines all fit inside
   // rectangle from 0,0 to 1,1
   double scalex = maxx - minx;
   double scaley = maxy - miny;
 
   for( l=0; l<num; l++ )
-    { 
+    {
       lines[l].x1 = (lines[l].x1 - minx) / scalex;
       lines[l].y1 = (lines[l].y1 - miny) / scaley;
       lines[l].x2 = (lines[l].x2 - minx) / scalex;
@@ -266,43 +293,43 @@ void stg_rotrects_normalize( stg_rotrect_t* rects, int num )
   double minx, miny, maxx, maxy;
   minx = miny = BILLION;
   maxx = maxy = -BILLION;
-  
+
   int r;
   for( r=0; r<num; r++ )
     {
       // test the origin of the rect
       if( rects[r].pose.x < minx ) minx = rects[r].pose.x;
-      if( rects[r].pose.y < miny ) miny = rects[r].pose.y;      
-      if( rects[r].pose.x > maxx ) maxx = rects[r].pose.x;      
+      if( rects[r].pose.y < miny ) miny = rects[r].pose.y;
+      if( rects[r].pose.x > maxx ) maxx = rects[r].pose.x;
       if( rects[r].pose.y > maxy ) maxy = rects[r].pose.y;
 
       // test the extremes of the rect
-      if( (rects[r].pose.x+rects[r].size.x)  < minx ) 
+      if( (rects[r].pose.x+rects[r].size.x)  < minx )
 	minx = (rects[r].pose.x+rects[r].size.x);
-      
-      if( (rects[r].pose.y+rects[r].size.y)  < miny ) 
+
+      if( (rects[r].pose.y+rects[r].size.y)  < miny )
 	miny = (rects[r].pose.y+rects[r].size.y);
-      
-      if( (rects[r].pose.x+rects[r].size.x)  > maxx ) 
+
+      if( (rects[r].pose.x+rects[r].size.x)  > maxx )
 	maxx = (rects[r].pose.x+rects[r].size.x);
-      
-      if( (rects[r].pose.y+rects[r].size.y)  > maxy ) 
+
+      if( (rects[r].pose.y+rects[r].size.y)  > maxy )
 	maxy = (rects[r].pose.y+rects[r].size.y);
     }
-  
+
   // now normalize all lengths so that the rects all fit inside
   // rectangle from 0,0 to 1,1
   double scalex = maxx - minx;
   double scaley = maxy - miny;
 
   for( r=0; r<num; r++ )
-    { 
+    {
       rects[r].pose.x = (rects[r].pose.x - minx) / scalex;
       rects[r].pose.y = (rects[r].pose.y - miny) / scaley;
       rects[r].size.x = rects[r].size.x / scalex;
       rects[r].size.y = rects[r].size.y / scaley;
     }
-}	
+}
 
 // returns an array of 4 * num_rects stg_line_t's
 stg_line_t* stg_rotrects_to_lines( stg_rotrect_t* rects, int num_rects )
@@ -310,7 +337,7 @@ stg_line_t* stg_rotrects_to_lines( stg_rotrect_t* rects, int num_rects )
   // convert rects to an array of lines
   int num_lines = 4 * num_rects;
   stg_line_t* lines = (stg_line_t*)calloc( sizeof(stg_line_t), num_lines );
-  
+
   int r;
   for( r=0; r<num_rects; r++ )
     {
@@ -318,23 +345,23 @@ stg_line_t* stg_rotrects_to_lines( stg_rotrect_t* rects, int num_rects )
       lines[4*r].y1 = rects[r].pose.y;
       lines[4*r].x2 = rects[r].pose.x + rects[r].size.x;
       lines[4*r].y2 = rects[r].pose.y;
-      
+
       lines[4*r+1].x1 = rects[r].pose.x + rects[r].size.x;;
       lines[4*r+1].y1 = rects[r].pose.y;
       lines[4*r+1].x2 = rects[r].pose.x + rects[r].size.x;
       lines[4*r+1].y2 = rects[r].pose.y + rects[r].size.y;
-      
+
       lines[4*r+2].x1 = rects[r].pose.x + rects[r].size.x;;
       lines[4*r+2].y1 = rects[r].pose.y + rects[r].size.y;;
       lines[4*r+2].x2 = rects[r].pose.x;
       lines[4*r+2].y2 = rects[r].pose.y + rects[r].size.y;
-      
+
       lines[4*r+3].x1 = rects[r].pose.x;
       lines[4*r+3].y1 = rects[r].pose.y + rects[r].size.y;
       lines[4*r+3].x2 = rects[r].pose.x;
       lines[4*r+3].y2 = rects[r].pose.y;
     }
-  
+
   return lines;
 }
 
@@ -344,10 +371,10 @@ stg_polygon_t* stg_polygons_from_rotrects( stg_rotrect_t* rects, size_t count,
 {
   stg_polygon_t* polys = stg_polygons_create( count );
   stg_point_t pts[4];
-  
+
   size_t r;
   for( r=0; r<count; r++ )
-    {  
+    {
       pts[0].x = rects[r].pose.x;
       pts[0].y = rects[r].pose.y;
       pts[1].x = rects[r].pose.x + rects[r].size.x;
@@ -356,7 +383,7 @@ stg_polygon_t* stg_polygons_from_rotrects( stg_rotrect_t* rects, size_t count,
       pts[2].y = rects[r].pose.y + rects[r].size.y;
       pts[3].x = rects[r].pose.x;
       pts[3].y = rects[r].pose.y + rects[r].size.y;
-      
+
       // copy these points in the polygon
       stg_polygon_set_points( &polys[r], pts, 4 );
 
@@ -364,7 +391,7 @@ stg_polygon_t* stg_polygons_from_rotrects( stg_rotrect_t* rects, size_t count,
       polys[r].bbox.x = width;
       polys[r].bbox.y = height;
     }
-  
+
   return polys;
 }
 
@@ -373,11 +400,11 @@ void stg_pose_sum( stg_pose_t* result, stg_pose_t* p1, stg_pose_t* p2 )
 {
   double cosa = cos(p1->a);
   double sina = sin(p1->a);
-  
+
   double tx = p1->x + p2->x * cosa - p2->y * sina;
   double ty = p1->y + p2->x * sina + p2->y * cosa;
   double ta = p1->a + p2->a;
-  
+
   result->x = tx;
   result->y = ty;
   result->a = ta;
@@ -411,23 +438,21 @@ void pb_set_pixel( GdkPixbuf* pb, int x, int y, uint8_t val )
     PRINT_WARN4( "pb_set_pixel coordinate %d,%d out of range (image dimensions %d by %d)", x, y, width, height );
 }
 
-// set all the pixels in a rectangle 
+// set all the pixels in a rectangle
 void pb_set_rect( GdkPixbuf* pb, int x, int y, int width, int height, uint8_t val )
 {
-  int pbwidth = gdk_pixbuf_get_width(pb);
-  int pbheight = gdk_pixbuf_get_height(pb);
   int bytes_per_sample = gdk_pixbuf_get_bits_per_sample (pb) / 8;
   int num_samples = gdk_pixbuf_get_n_channels(pb);
 
   int a, b;
   for( a = y; a < y+height; a++ )
     for( b = x; b < x+width; b++ )
-      {	
+      {
 	// zeroing
 	guchar* pix = pb_get_pixel( pb, b, a );
 	memset( pix, val, num_samples * bytes_per_sample );
       }
-}  
+}
 
 // returns TRUE if any channel in the pixel is non-zero
 gboolean pb_pixel_is_set( GdkPixbuf* pb, int x, int y, int threshold )
@@ -443,7 +468,7 @@ gboolean pb_pixel_is_set( GdkPixbuf* pb, int x, int y, int threshold )
 }
 
 
-stg_polygon_t* stg_polygons_from_image_file(  const char* filename, 
+stg_polygon_t* stg_polygons_from_image_file(  const char* filename,
 					     size_t* count )
 {
   stg_polygon_t* polys;
@@ -451,13 +476,13 @@ stg_polygon_t* stg_polygons_from_image_file(  const char* filename,
   int rect_count = 0;
 
   int width, height;
-  if( stg_rotrects_from_image_file( filename,  
+  if( stg_rotrects_from_image_file( filename,
 				    &rects,
 				    &rect_count,
 				    &width, &height ) )
     {
       PRINT_ERR1( "failed to load rects from image file \"%s\"",
-		  filename );      
+		  filename );
       return NULL;
     }
 
@@ -470,46 +495,43 @@ stg_polygon_t* stg_polygons_from_image_file(  const char* filename,
   return(polys);
 }
 
-stg_polyline_t* stg_polylines_from_image_file( const char* filename, 
+stg_polyline_t* stg_polylines_from_image_file( const char* filename,
 					       size_t* num )
 {
-  // TODO: make this a parameter
-  const int threshold = 127;
-
   GError* err = NULL;
   GdkPixbuf* pb = gdk_pixbuf_new_from_file( filename, &err );
 
   if( err )
     {
       fprintf( stderr, "\nError loading bitmap: %s\n", err->message );
-      return 1; // error
+      return (stg_polyline_t*)1; // error
     }
-  
+
   // this should be ok as no error was reported
   assert( pb );
 
   stg_polyline_t* lines = NULL;
   size_t lines_count = 0;
-  
+
   int img_width = gdk_pixbuf_get_width(pb);
   int img_height = gdk_pixbuf_get_height(pb);
-  
+
   int y, x;
   for(y = 0; y < img_height; y++)
     for(x = 0; x < img_width; x++)
       {
 	// TODO!
-      }	
-  
+      }
+
   // free the image data
   gdk_pixbuf_unref( pb );
-  
+
   if( num ) *num = lines_count;
   return lines;
-}				   
+}
 
-int stg_rotrects_from_image_file( const char* filename, 
-				  stg_rotrect_t** rects, 
+int stg_rotrects_from_image_file( const char* filename,
+				  stg_rotrect_t** rects,
 				  int* rect_count,
 				  int* widthp, int* heightp )
 {
@@ -524,19 +546,19 @@ int stg_rotrects_from_image_file( const char* filename,
       fprintf( stderr, "\nError loading bitmap: %s\n", err->message );
       return 1; // error
     }
-  
+
   // this should be ok as no error was reported
   assert( pb );
-  
-  
+
+
 #ifdef DEBUG
   printf( "image \"%s\" channels:%d bits:%d alpha:%d "
 	  "width:%d height:%d rowstride:%d pixels:%p\n",
-	  
+
 	  filename,
 	  gdk_pixbuf_get_n_channels(pb),
 	  gdk_pixbuf_get_bits_per_sample(pb),
-	  gdk_pixbuf_get_has_alpha(pb),	      
+	  gdk_pixbuf_get_has_alpha(pb),
 	  gdk_pixbuf_get_width(pb),
 	  gdk_pixbuf_get_height(pb),
 	  gdk_pixbuf_get_rowstride(pb),
@@ -545,14 +567,14 @@ int stg_rotrects_from_image_file( const char* filename,
 
   *rect_count = 0;
   *rects = NULL;
-  
+
   int img_width = gdk_pixbuf_get_width(pb);
   int img_height = gdk_pixbuf_get_height(pb);
-  
+
   // if the caller wanted to know the dimensions
   if( widthp ) *widthp = img_width;
   if( heightp ) *heightp = img_height;
-  
+
   int y, x;
   for(y = 0; y < img_height; y++)
     {
@@ -561,60 +583,58 @@ int stg_rotrects_from_image_file( const char* filename,
 	  // skip blank (white) pixels
 	  if(  pb_pixel_is_set( pb,x,y, threshold) )
 	    continue;
-	  
+
 	  // a rectangle starts from this point
 	  int startx = x;
 	  int starty = y;
-	  int height = img_height; // assume full height for starters
-	  
-	  // grow the width - scan along the line until we hit an empty (white) pixel
-	  for( ; x < img_width &&  ! pb_pixel_is_set(pb,x,y,threshold); x++ )
-	    {
-	      // handle horizontal cropping
-	      //double ppx = x * sx; 
-	      //if (ppx < this->crop_ax || ppx > this->crop_bx)
-	      //continue;
-	      
-	      // look down to see how large a rectangle below we can make
-	      int yy  = y;
-	      while( ! pb_pixel_is_set(pb,x,yy,threshold) && (yy < img_height-1) )
-		{ 
-		  // handle vertical cropping
-		  //double ppy = (this->image->height - yy) * sy;
-		  //if (ppy < this->crop_ay || ppy > this->crop_by)
-		  //continue;
-		  
-		  yy++; 
-		} 	      
+	  int height = img_height - y; // assume full height for starters
+
+          // grow the width - scan along the line until we hit an empty (white) pixel
+          for( ; x < img_width &&  ! pb_pixel_is_set(pb,x,y,threshold); x++ )
+          {
+            // handle horizontal cropping
+            //double ppx = x * sx;
+            //if (ppx < this->crop_ax || ppx > this->crop_bx)
+            //continue;
+
+            // look down to see how large a rectangle below we can make
+            int yy;
+            for (yy = y + 1;  yy < y + height; ++yy)
+            {
+              // handle vertical cropping
+              //double ppy = (this->image->height - yy) * sy;
+              //if (ppy < this->crop_ay || ppy > this->crop_by)
+              //continue;
+              if (pb_pixel_is_set(pb,x,yy,threshold)) // found a full (non-white) pixel --> smaller height!
+                {
+                  height = yy - y;
+                  break;
+                }
+            }
+          }
 
-	      // now yy is the depth of a line of non-zero pixels
-	      // downward we store the smallest depth - that'll be the
-	      // height of the rectangle
-	      if( yy-y < height ) height = yy-y; // shrink the height to fit
-	    } 
-	  
 	  // whiten the pixels we have used in this rect
 	  pb_set_rect( pb, startx, starty, x-startx, height, 0xFF );
-	  
+
 	  // add this rectangle to the array
 	  (*rect_count)++;
 	  *rects = (stg_rotrect_t*)
 	    realloc( *rects, *rect_count * sizeof(stg_rotrect_t) );
-	  
+
 	  stg_rotrect_t *latest = &(*rects)[(*rect_count)-1];
 	  latest->pose.x = startx;
 	  latest->pose.y = starty;
 	  latest->pose.a = 0.0;
 	  latest->size.x = x - startx;
 	  latest->size.y = height;
-	  
-	  //printf( "rect %d (%.2f %.2f %.2f %.2f %.2f\n", 
-	  //  *rect_count, 
-	  //  latest->x, latest->y, latest->a, latest->w, latest->h ); 
-	  
+
+	  //printf( "rect %d (%.2f %.2f %.2f %.2f %.2f\n",
+	  //  *rect_count,
+	  //  latest->x, latest->y, latest->a, latest->w, latest->h );
+
 	}
     }
-  
+
   // free the image data
   gdk_pixbuf_unref( pb );
 
@@ -624,11 +644,11 @@ int stg_rotrects_from_image_file( const char* filename,
   int r;
   for( r=0; r< *rect_count; r++ )
     {
-      stg_rotrect_t *rect = &(*rects)[r]; 
+      stg_rotrect_t *rect = &(*rects)[r];
       rect->pose.y = img_height - rect->pose.y;
       rect->size.y = -rect->size.y;
     }
-  
+
 
   return 0; // ok
 }
@@ -651,12 +671,14 @@ void stg_points_destroy( stg_point_t* pts )
 stg_polygon_t* stg_polygons_create( int count )
 {
   stg_polygon_t* polys = (stg_polygon_t*)calloc( count, sizeof(stg_polygon_t));
-  
+
   // each polygon contains an array of points
   int p;
   for( p=0; p<count; p++ )
     polys[p].points = g_array_new( FALSE, TRUE, sizeof(stg_point_t));
 
+  //polys[p].color = 0xAA00FF00;
+
   return polys;
 }
 
@@ -664,11 +686,14 @@ stg_polygon_t* stg_polygons_create( int count )
 void stg_polygons_destroy( stg_polygon_t* p, size_t count )
 {
   int c;
+
+  assert(p != NULL);
+
   for( c=0; c<count; c++ )
     if( p[c].points )
       g_array_free( p[c].points, TRUE );
-  
-  free( p );      
+
+  free( p );
 }
 
 stg_polygon_t* stg_unit_polygon_create( void )
@@ -681,17 +706,17 @@ stg_polygon_t* stg_unit_polygon_create( void )
   pts[2].x = 1;
   pts[2].y = 1;
   pts[3].x = 0;
-  pts[3].y = 1;  
-  
+  pts[3].y = 1;
+
   stg_polygon_t* poly = stg_polygons_create(1);
-  stg_polygon_set_points( poly, pts, 4 );  
+  stg_polygon_set_points( poly, pts, 4 );
   return poly;
 }
 
 //////////////////////////////////////////////////////////////////////////
 // scale an array of polygons so they fit in a rectangle of size
 // [width] by [height], with the origin in the center of the rectangle
-void stg_polygons_normalize( stg_polygon_t* polys, int num, 
+void stg_polygons_normalize( stg_polygon_t* polys, int num,
 			     double width, double height )
 {
   if( num == 0 )
@@ -701,7 +726,7 @@ void stg_polygons_normalize( stg_polygon_t* polys, int num,
   double minx, miny, maxx, maxy;
   minx = miny = BILLION;
   maxx = maxy = -BILLION;
-  
+
   int l;
   for( l=0; l<num; l++ ) // examine all the polygons
     {
@@ -719,7 +744,7 @@ void stg_polygons_normalize( stg_polygon_t* polys, int num,
 	  assert( ! isnan( pt->y ) );
 	}
     }
-  
+
   //minx = 0;
   //miny = 0;
   // maxx = polys[0].bbox.x;
@@ -732,15 +757,15 @@ void stg_polygons_normalize( stg_polygon_t* polys, int num,
 
   //double scalex = polys[0].bbox.x;
   //double scaley = polys[0].bbox.y;
-  
+
   for( int l=0; l<num; l++ ) // scale each polygon
-    { 
+    {
       // scale all the points in the polygon
       int p;
       for( p=0; p<polys[l].points->len; p++ )
 	{
 	  stg_point_t* pt = &g_array_index( polys[l].points, stg_point_t, p);
-	  
+
 	  pt->x = ((pt->x - minx) / scalex * width) - width/2.0;
 	  pt->y = ((pt->y - miny) / scaley * height) - height/2.0;
 
@@ -753,7 +778,7 @@ void stg_polygons_normalize( stg_polygon_t* polys, int num,
 void stg_polygon_print( stg_polygon_t* poly )
 {
   printf( "polygon: %d pts : ", poly->points->len );
-  
+
   int i;
   for(i=0;i<poly->points->len;i++)
     {
@@ -766,11 +791,11 @@ void stg_polygon_print( stg_polygon_t* poly )
 void stg_polygons_print( stg_polygon_t* polys, unsigned int count )
 {
   printf( "polygon array (%d polys)\n", count );
-  
+
   int i;
   for( i=0; i<count; i++ )
     {
-      printf( "[%d] ", i ); 
+      printf( "[%d] ", i );
       stg_polygon_print( &polys[i] );
     }
 }
@@ -796,7 +821,7 @@ void stg_polygons_print( stg_polygon_t* polys, unsigned int count )
 void stg_polygon_set_points( stg_polygon_t* poly, stg_point_t* pts, size_t count )
 {
   assert( poly );
-  
+
   g_array_set_size( poly->points, 0 );
   g_array_append_vals( poly->points, pts, count );
 }
